﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Metadata;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PuzzlePlayer_Namespace
{
    /*
     * This abstract class can be used to implement any kind of puzzle
     * One thing that is common for all puzzles is that a empty space is equal to -1
     */
    public abstract class Board
    {
        protected const int emptySpace = -1; // for every puzzle -1 represents a empty space

        // a property for getting and setting the boardsstate. The boardstate should only be setted if the imputed int[,] is valid.
        public int[,] boardState
        {
            get { return boardState; }
            set { if (IsBoardValid(value) && Solve(value) != null) boardState = value; } // checks if the board is valid and solvable before setting the variable.
        }

        // a virtual methode for solving the whole board. It uses the abstract SolveStep methode untill the whole board is solved
        public virtual int[,] Solve(int[,] boardToSolve)
        {
            // two variables for storing the result and the next solveStep
            int[,] result = boardToSolve;
            int[,] step;

            // keep looping until the SolveStep returns null
            // if SolveStep returns null that could mean that either the whole board is solved or it is imposible to solve the board
            while ((step = SolveStep(result)) != null)
            {
                result = step;
            }

            // check if the whole board is filled
            // if not then the methode will return null because it is imposible to solve the board
            for (int i = 0; i < result.GetLength(0); i++)
                for (int j = 0; j < result.GetLength(1); j++)
                    if (result[i, j] == emptySpace)
                        return null;

            return result;
        }

        // abstract methode for solving one step
        protected abstract int[,] SolveStep(int[,] currentBoardState);

        // abstract methode for generating a random board
        public abstract void Generate();

        // abstract methode for checking if a imputed boardstate is valid
        public abstract bool IsBoardValid(int[,] boardToCheck);
    }
}
